import latch.Latch;
import latch.LatchSupport;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.dbi.EnvironmentImpl;

/**
 * adds mapTreeRootLatch and triggerLatch
 */
public aspect EnvironmentImplExtension {

	private Latch EnvironmentImpl.mapTreeRootLatch;

	before(EnvironmentImpl envImpl) : execution(EnvironmentImpl.new(..)) && target(envImpl) {
		// before constructor
		envImpl.mapTreeRootLatch = LatchSupport.makeLatch("MapTreeRoot",
				envImpl);
	}

	pointcut latchedMethods(EnvironmentImpl envImpl):(
			execution(void EnvironmentImpl.logMapTreeRoot()) ||
			execution(void EnvironmentImpl.rewriteMapTreeRoot(long)) ||
			execution(void EnvironmentImpl.readMapTreeFromLog(long))
		) && target(envImpl);

	/**
	 * Log the map tree root and save the LSN.
	 */
	Object around(EnvironmentImpl envImpl) throws DatabaseException : 
		latchedMethods(envImpl){
		envImpl.mapTreeRootLatch.acquire();
		try {
			return proceed(envImpl);
		} finally {
			envImpl.mapTreeRootLatch.release();
		}
	}

	boolean around() : call(boolean EnvironmentImpl.hook_assertLatchClean()) {
		boolean clean = proceed();
		if (LatchSupport.countLatchesHeld() > 0) {
			clean = false;
			System.out.println("Some latches held at env close.");
			LatchSupport.dumpLatchesHeld();
		}
		return clean;
	}

	after() :
		execution(void hook_afterDoClose(StringBuffer)) {
		LatchSupport.clearNotes();
	}

}
