import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.config.BooleanConfigParam;
import com.sleepycat.je.config.EnvironmentParams;
import com.sleepycat.je.dbi.DbConfigManager;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.log.SyncedLogManager;
import com.sleepycat.je.txn.LatchedLogManager;

public privileged aspect FairLatchesExtension {
	private static boolean EnvironmentImpl.fairLatches;// true if user wants

	// fair latches

	public static final BooleanConfigParam ENV_FAIR_LATCHES = new BooleanConfigParam(
			"je.env.fairLatches",
			false, // default
			false, // mutable
			"# If true, use latches instead of synchronized blocks to\n"
					+ "# implement the lock table and log write mutexes. Latches require\n"
					+ "# that threads queue to obtain the mutex in question and\n"
					+ "# therefore guarantee that there will be no mutex starvation, but \n"
					+ "# do incur a performance penalty. Latches should not be necessary in\n"
					+ "# most cases, so synchronized blocks are the default. An application\n"
					+ "# that puts heavy load on JE with threads with different thread\n"
					+ "# priorities might find it useful to use latches.  In a Java 5 JVM,\n"
					+ "# where java.util.concurrent.locks.ReentrantLock is used for the\n"
					+ "# latch implementation, this parameter will determine whether they\n"
					+ "# are 'fair' or not.  This parameter is 'static' across all\n"
					+ "# environments.\n");

	after(DbConfigManager configManager) throws DatabaseException : 
		call(void hook_readProperties(DbConfigManager)) && args(configManager){
		EnvironmentImpl.fairLatches = configManager
				.getBoolean(ENV_FAIR_LATCHES);

	}

	Object around(EnvironmentImpl env, boolean isReadOnly)
			throws DatabaseException : 
		call(SyncedLogManager.new(EnvironmentImpl, boolean)) && this(env) 
		&& args(EnvironmentImpl,isReadOnly) 
	{
		if (env.fairLatches) {
			return new LatchedLogManager(env, isReadOnly);
		} else {
			return proceed(env, isReadOnly);
		}
	}

	public static boolean EnvironmentImpl.getFairLatches() {
		return fairLatches;
	}

}
