package SubFeature_syncLockManager;
import latch.Latch;
import latch.LatchStats;
import latch.LatchSupport;

import com.sleepycat.je.LockStats;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.txn.LockManager;
import com.sleepycat.je.DatabaseException;

public privileged aspect LockManagerExtension {

	after(LockManager lockManager, EnvironmentImpl envImpl) : 
		execution(LockManager.new(EnvironmentImpl)) && target(lockManager) && args(envImpl)
	{
		lockManager.lockTableLatches = new Latch[lockManager.nLockTables];
		for (int i = 0; i < lockManager.nLockTables; i++) {
			lockManager.lockTableLatches[i] = LatchSupport.makeLatch(
					"Lock Table " + i, envImpl);
		}
	}
	
	after (LockManager lockManager, LockStats stats) : 
		call(void hook_lockStatLatches(LockStats)) && args(stats) && this(lockManager) {
		for (int i = 0; i < lockManager.nLockTables; i++) {
			LatchStats latchStats = (LatchStats) lockManager.lockTableLatches[i]
					.getLatchStats();
			stats.accumulateLockTableLatchStats(latchStats);
		}
	}
	

	public Latch[] LockManager.lockTableLatches;
	
	/**
	 * LockTable latch stats.
	 */
	private LatchStats LockStats.lockTableLatchStats;

	
	/**
	 * Internal use only.
	 */
	public void LockStats.accumulateLockTableLatchStats(LatchStats latchStats) {
		if (lockTableLatchStats == null) {
			lockTableLatchStats = latchStats;
			return;
		}
	
		lockTableLatchStats.nAcquiresNoWaiters += latchStats.nAcquiresNoWaiters;
		lockTableLatchStats.nAcquiresSelfOwned += latchStats.nAcquiresSelfOwned;
		lockTableLatchStats.nAcquiresUpgrade += latchStats.nAcquiresUpgrade;
		lockTableLatchStats.nAcquiresWithContention += latchStats.nAcquiresWithContention;
		lockTableLatchStats.nAcquireNoWaitSuccessful += latchStats.nAcquireNoWaitSuccessful;
		lockTableLatchStats.nAcquireNoWaitUnsuccessful += latchStats.nAcquireNoWaitUnsuccessful;
		lockTableLatchStats.nAcquireSharedSuccessful += latchStats.nAcquireSharedSuccessful;
	}

	before (StringBuffer sb, LockStats lockStats) : 
		call(String StringBuffer.toString()) && 
		withincode(String LockStats.toString()) && target(sb) && 
		this(lockStats) 
	{
		sb.append("lockTableLatch:\n").append(lockStats.lockTableLatchStats);
	}


	void around (LockManager lockManager, int idx) throws DatabaseException : 
		execution(void LockManager.hook_dumpToStringLatched(StringBuffer,int)) 
		&& args(StringBuffer,idx) && this(lockManager) {
		lockManager.lockTableLatches[idx].acquire();
		try {
			proceed(lockManager, idx);
		} finally {
			lockManager.lockTableLatches[idx].release();
		}		
	}

}
