import latch.Latch;
import latch.LatchSupport;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.StatsConfig;
import com.sleepycat.je.dbi.EnvironmentImpl;
import com.sleepycat.je.txn.Txn;
import com.sleepycat.je.txn.TxnManager;

/**
 * simple maintainable homogeous pointcut and advice!
 */
public privileged aspect TxnManagerExtension {
	private Latch TxnManager.allTxnLatch;

	after(EnvironmentImpl env, TxnManager mgr) : 
		execution(TxnManager.new(EnvironmentImpl)) && args(env) && this(mgr) 
	{
		mgr.allTxnLatch = LatchSupport.makeLatch(TxnManager.DEBUG_NAME, env);
	}

	pointcut latchedMethods(TxnManager mgr) :
		(execution(void TxnManager.registerTxn(Txn)) || 
		execution(void TxnManager.unRegisterTxn(Txn, boolean)) ||
		execution(long TxnManager.getFirstActiveLsn()) || 
		execution(* TxnManager.txnStat(StatsConfig))) && this(mgr);

	Object around(TxnManager mgr) throws DatabaseException : latchedMethods(mgr){
		mgr.allTxnLatch.acquire();
		try {
			return proceed(mgr);
		} finally {
			mgr.allTxnLatch.release();
		}
	}

}
