import java.util.Set;

import latch.LatchSupport;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.dbi.INList;
import com.sleepycat.je.dbi.MemoryBudget;
import com.sleepycat.je.dbi.SortedLSNTreeWalker;
import com.sleepycat.je.recovery.RecoveryManager;
import com.sleepycat.je.tree.TreeLocation;
import com.sleepycat.je.txn.Txn;

public aspect VariousSmallLatchExtensions {

	Object around(TreeLocation location) throws DatabaseException :
		call(void RecoveryManager.undo(..)) && args(*,*,location,..) &&
		withincode(* Txn.undo()) {

		try {
			return proceed(location);
		} finally {
			if (location.bin != null) {
				location.bin.releaseLatchIfOwner();
			}
		}
	}

	long around(INList inList) throws DatabaseException : 
		call(long MemoryBudget.hook_getSize(INList)) && args(inList)
	{
		inList.latchMajor();
		try {
			return proceed(inList);
		} finally {
			inList.releaseMajorLatch();
		}
	}

	after():execution(void MemoryBudget.initCacheMemoryUsage()){
		assert LatchSupport.countLatchesHeld() == 0;
	}

	void around(INList inList) throws DatabaseException :
		execution(void SortedLSNTreeWalker.hook_findInternal(INList, Set)) && args(inList,Set){
		inList.latchMajor();
		try {
			/* consolidate the INList first. */
			inList.latchMinorAndDumpAddedINs();//

			proceed(inList);
		} finally {
			inList.releaseMajorLatch();
		}

	}
}
