package com.sleepycat.je.evictor;

import latch.LatchSupport;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.dbi.INList;
import com.sleepycat.je.evictor.Evictor.ScanIterator;
import com.sleepycat.je.tree.IN;

public aspect EvictorExtension {
	long around(IN targetIN) throws DatabaseException :
		execution(long Evictor.evict(INList, IN, ScanIterator)) && 
		args(INList,targetIN,ScanIterator) 
	{
		if (targetIN.latchNoWait(false)) {
			try {
				return proceed(targetIN);
			} finally {
				targetIN.releaseLatchIfOwner();
			}
		} else
			return 0;
	}

	long around(INList inList) throws DatabaseException :
		execution(long Evictor.hook_evictInternal(INList, long)) && args(inList,long) {
		inList.latchMajor();

		long result = 0;
		try {
			result = proceed(inList);
		} finally {
			inList.releaseMajorLatch();
		}

		assert LatchSupport.countLatchesHeld() == 0 : "latches held = "
		+ LatchSupport.countLatchesHeld();
		return result;
	}

	Object around(IN parent) throws DatabaseException :
		execution(long Evictor.evictIN(IN, IN, int, INList,	ScanIterator, boolean)) && 
		args(IN, parent, int, INList,	ScanIterator, boolean) 
	{
		try {
			assert parent.isLatchOwner();
			return proceed(parent);
		} finally {
			parent.releaseLatch();
		}
	}
}
