package com.sleepycat.je.log;

import latch.Latch;
import latch.LatchSupport;

import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.dbi.EnvironmentImpl;

public privileged aspect LogBufferExtension {

	// protected not allowed -> needed to make public
	public Latch LogBuffer.readLatch;

	before(LogBuffer dt, EnvironmentImpl env) : 
		execution(LogBuffer.new(int, EnvironmentImpl)) && this(dt) && args(int, env) {
		dt.readLatch = LatchSupport.makeLatch(LogBuffer.DEBUG_NAME, env);

	}

	Object around(LogBuffer lb) throws DatabaseException :
		(execution(void LogBuffer.registerLsn(long)) ||
				execution(void LogBuffer.reinit()))
		&& target(lb) 
	{
		lb.readLatch.acquire();
		try {
			return proceed(lb);
		} finally {
			lb.readLatch.release();
		}
	}

	boolean around(LogBuffer lb) throws DatabaseException :
		execution(boolean LogBuffer.containsLsn(long) ) && target(lb) 
	{
		lb.readLatch.acquire();
		boolean r = proceed(lb);
		if (!r)
			lb.readLatch.release();
		return r;
	}
	
	after(LogBuffer dt) throws DatabaseException : 
		execution(void LogBuffer.release()) && this(dt) {
		dt.readLatch.release();
	}
	
	/**
	 * When modifying the buffer, acquire the readLatch. Call release() to
	 * release the latch. Note that containsLsn() acquires the latch for
	 * reading.
	 */
	 public void LogBuffer.latchForWrite() throws DatabaseException {

		readLatch.acquire();
	}
	 
	 
	
	 

}
