package cursor;

import latch.LatchSupport;

import com.sleepycat.je.Cursor;
import com.sleepycat.je.DatabaseException;
import com.sleepycat.je.OperationStatus;
import com.sleepycat.je.dbi.CursorImpl;
import com.sleepycat.je.dbi.CursorImpl.KeyChangeStatus;
import com.sleepycat.je.tree.BIN;
import com.sleepycat.je.tree.DBIN;

public privileged aspect CursorExtension {

	before(CursorImpl dup) throws DatabaseException:
		call(OperationStatus CursorImpl.delete()) && target(dup) && 
		withincode(OperationStatus Cursor.deleteNoNotify()) {
		dup.latchBINs();
	}

	before(CursorImpl original, CursorImpl dup) throws DatabaseException:
		execution(void Cursor.hook_afterDeletion(CursorImpl, CursorImpl)) && args(original, dup) 
	{
		if (original != null) {
			original.releaseBINs();
		}
		if (dup != null) {
			dup.releaseBINs();
		}
	}

	before(CursorImpl original) throws DatabaseException:
		execution(void Cursor.hook_afterPut(CursorImpl)) && args(original) 
	{
		if (original != null) {
			original.releaseBINs();
		}
	}

	after():execution(void Cursor.hook_assertNoLatches()){
		assert LatchSupport.countLatchesHeld() == 0 :
			LatchSupport.latchesHeldToString();
	}

	after():execution(void Cursor.hook_assertOneLatch()){
		assert LatchSupport.countLatchesHeld() == 0 :
			LatchSupport.latchesHeldToString();
	}

	KeyChangeStatus around(Cursor cursor, CursorImpl dup)
			throws DatabaseException:
		execution(* Cursor.searchInternal(..)) && args(dup,..) && this(cursor){
		KeyChangeStatus result = null;
		try {
			result = proceed(cursor, dup);
		} finally {

			/*
			 * searchAndPosition returns with the target BIN latched, so it is
			 * the responsibility of this method to make sure the latches are
			 * released.
			 */
			cursor.cursorImpl.releaseBINs();
			if (result.status != OperationStatus.SUCCESS
					&& dup != cursor.cursorImpl) {
				dup.releaseBINs();
			}
		}
		return result;

	}

	Object around(CursorImpl origCursor) throws DatabaseException : 
		(execution(boolean Cursor.hook_checkInternal(CursorImpl, DBIN, boolean, BIN)) ||
				execution(boolean Cursor.hook_checkInternal2(CursorImpl, DBIN, boolean, BIN))) && 
		args(origCursor,*,*,*) 
	{
		origCursor.latchBINs();
		Object r=proceed(origCursor);
		origCursor.releaseBINs();
		return r;
	}

}
